import reclass_rs

from pathlib import Path


def test_nodeinfo_n1():
    r = reclass_rs.Reclass(
        nodes_path="./tests/inventory/nodes", classes_path="./tests/inventory/classes"
    )
    n = r.nodeinfo("n1")
    npath = Path("./tests/inventory/nodes/n1.yml").resolve()
    assert n.__reclass__.uri == f"yaml_fs://{npath}"
    assert n.applications == ["app1", "app2"]
    assert n.classes == ["cls1", "cls2"]
    assert n.parameters == {
        "_reclass_": {
            "environment": "base",
            "name": {
                "full": "n1",
                "parts": ["n1"],
                "path": "n1",
                "short": "n1",
            },
        },
        "foo": {"foo": "foo", "bar": "cls2", "baz": "cls1"},
        "bar": {"foo": "foo"},
    }


def test_nodeinfo_n1_meta_symlink():
    r = reclass_rs.Reclass(
        nodes_path="./tests/inventory/targets", classes_path="./tests/inventory/classes"
    )
    n = r.nodeinfo("n1")
    npath = Path("./tests/inventory/targets/n1.yml").absolute()
    assert n.__reclass__.uri == f"yaml_fs://{npath}"


def test_nodeinfo_n2():
    r = reclass_rs.Reclass(
        nodes_path="./tests/inventory/nodes", classes_path="./tests/inventory/classes"
    )
    n = r.nodeinfo("n2")
    assert n.applications == []
    assert n.classes == ["nested.cls2", "nested.cls1"]
    assert n.parameters == {
        "_reclass_": {
            "environment": "base",
            "name": {
                "full": "n2",
                "parts": ["n2"],
                "path": "n2",
                "short": "n2",
            },
        },
        "foo": {"foo": "nested.cls1", "bar": "n2"},
        "bar": "bar",
    }


def test_nodeinfo_n3():
    r = reclass_rs.Reclass(
        nodes_path="./tests/inventory/nodes", classes_path="./tests/inventory/classes"
    )
    n = r.nodeinfo("n3")
    assert n.applications == []
    assert n.classes == ["cls4", "cls5", "cls6", "cls3"]
    assert n.parameters == {
        "_reclass_": {
            "environment": "base",
            "name": {
                "full": "n3",
                "parts": ["n3"],
                "path": "n3",
                "short": "n3",
            },
        },
        "cluster": {"name": "c-test-cluster-1234"},
        "openshift": {
            "infraID": "c-test-cluster-1234-xlk3f",
            "clusterID": "2888efd2-8a1b-4846-82ec-3a99506e2c70",
            "baseDomain": "c-test-cluster-1234.example.org",
            "appsDomain": "apps.c-test-cluster-1234.example.org",
            "apiURL": "api.c-test-cluster-1234.example.org",
            "ssh_key": "",
        },
    }


def test_nodeinfo_n4():
    r = reclass_rs.Reclass(
        nodes_path="./tests/inventory/nodes", classes_path="./tests/inventory/classes"
    )
    n = r.nodeinfo("n4")
    assert n.applications == []
    assert n.classes == ["cls8", "${qux}", "cls7"]
    assert n.parameters == {
        "_reclass_": {
            "environment": "base",
            "name": {
                "full": "n4",
                "parts": ["n4"],
                "path": "n4",
                "short": "n4",
            },
        },
        "qux": "cls1",
        "foo": {"foo": "cls1", "bar": "cls1", "baz": "cls1"},
    }



import reclass_rs

expected_classes = {
    "${cls9}": ["n15"],
    "${tenant}.${cluster}": ["n16"],
    "\\${baz}": ["n17"],
    "${qux}": ["n4"],
    "app1": ["n12"],
    "app2": ["n13"],
    "cls1": ["n1"],
    "cls2": ["n1"],
    "cls3": ["n3"],
    "cls4": ["n3"],
    "cls5": ["n3"],
    "cls6": ["n3"],
    "cls7": ["n4"],
    "cls8": ["n4"],
    "cls9": [
        "n10",
        "n12",
        "n13",
        "n14",
        "n18",
        "n5",
        "n6",
        "n7",
        "n9",
    ],
    "cls9_meta": ["n15"],
    "cls10": ["n13", "n5", "n9"],
    "cls11": ["n6"],
    "cls12": ["n9"],
    "cls13": ["n14"],
    "cls14": ["n23"],
    "cls15": ["n23"],
    "cluster.${dist}": ["n19"],
    "cluster.default": ["n19"],
    "cluster.facts": ["n19"],
    "cluster.global": ["n19"],
    "config": ["n16"],
    "foo-indirect": ["n20"],
    "nested.a": ["n8"],
    "nested.a_sub": ["n8", "n9"],
    "nested.b": ["n10"],
    "nested.cls1": ["n2"],
    "nested.cls2": ["n2"],
    "nonexisting": ["n18"],
    "yaml-anchor": ["n21"],
}

expected_applications = {
    "app1": ["n1"],
    "app2": ["n1"],
    "a": ["n12", "n13"],
    "b": ["n13"],
    "c": ["n12"],
    "d": ["n13"],
}

expected_nodes = set([f"n{i}" for i in range(1, 24)])


def test_inventory():
    r = reclass_rs.Reclass(
        nodes_path="./tests/inventory/nodes",
        classes_path="./tests/inventory/classes",
        ignore_class_notfound=True,
    )
    inv = r.inventory()

    assert set(inv.nodes.keys()) == expected_nodes

    assert inv.classes == expected_classes

    assert inv.applications == expected_applications


def test_inventory_as_dict():
    r = reclass_rs.Reclass(
        nodes_path="./tests/inventory/nodes",
        classes_path="./tests/inventory/classes",
        ignore_class_notfound=True,
    )
    inv = r.inventory().as_dict()

    assert set(inv["nodes"].keys()) == expected_nodes

    assert inv["classes"] == expected_classes

    assert inv["applications"] == expected_applications

    assert "__reclass__" in inv
    assert set(inv["__reclass__"].keys()) == set(["timestamp"])



import reclass_rs
import pytest
import os


def test_import_new():
    r = reclass_rs.Reclass(
        "./tests/inventory/targets", "./tests/inventory/classes", True
    )
    assert r is not None
    assert r.nodes_path == "./tests/inventory/targets"
    assert r.classes_path == "./tests/inventory/classes"
    assert r.ignore_class_notfound


def test_import_raises():
    with pytest.raises(ValueError) as exc:
        r = reclass_rs.Reclass("./foo", "./bar")

    assert "Error while discovering nodes" in str(exc.value)
